import importlib


def get_workbook_impl():
    try:
        provider = importlib.import_module('djaxei.providers.xlsxwriter_provider')
        return provider.WorkbookImpl
    except:
        pass

    try:
        provider = importlib.import_module('djaxei.providers.xlwt_provider')
        return provider.WorkbookImpl
    except:
        pass


from django.contrib import messages
from django.contrib.admin import helpers
from django.contrib.admin.utils import get_deleted_objects, model_ngettext
from django.core.exceptions import PermissionDenied
from django.db import router
from django.template.response import TemplateResponse
from django.utils.encoding import force_text
from django.utils.translation import ugettext as _, ugettext_lazy


def replace_selected_validator(modeladmin, request, queryset):
    # Check that the user has delete permission for the actual model
    if not modeladmin.has_delete_permission(request):
        raise PermissionDenied
    return None


def replace_selected_factory(breadcrumb=None, validate_queryset=None):
    def replace_selected(modeladmin, request, queryset):
        """
        Default action which deletes the selected objects.

        This action first displays a confirmation page which shows all the
        deletable objects, or, if the user has no permission one of the related
        childs (foreignkeys), a "permission denied" message.

        Next, it deletes all selected objects and redirects back to the change list.
        """
        opts = modeladmin.model._meta
        app_label = opts.app_label

        if not modeladmin.has_delete_permission(request):
            raise PermissionDenied

        if validate_queryset:
            validation_error_message = validate_queryset(modeladmin, request, queryset)
            if validation_error_message:
                modeladmin.message_user(request, validation_error_message, messages.WARNING)
                return None

        using = router.db_for_write(modeladmin.model)

        # Populate deletable_objects, a data structure of all related objects that
        # will also be deleted.
        deletable_objects, model_count, perms_needed, protected = get_deleted_objects(
            queryset, opts, request.user, modeladmin.admin_site, using)

        # The user has already confirmed the deletion.
        # Do the deletion and return a None to display the change list view again.
        if request.POST.get('post') and not protected:
            if perms_needed:
                raise PermissionDenied
            n = queryset.count()
            if n:
                for obj in queryset:
                    obj_display = force_text(obj)
                    modeladmin.log_deletion(request, obj, obj_display)
                queryset.delete()
                modeladmin.message_user(request, _("Successfully deleted %(count)d %(items)s.") % {
                    "count": n, "items": model_ngettext(modeladmin.opts, n)
                }, messages.SUCCESS)
            # Return None to display the change list page again.
            return None

        if len(queryset) == 1:
            objects_name = force_text(opts.verbose_name)
        else:
            objects_name = force_text(opts.verbose_name_plural)

        if perms_needed or protected:
            title = _("Cannot delete %(name)s") % {"name": objects_name}
        else:
            title = _("Are you sure you want to replace those records with the content of the spreadsheet?")

        context = dict(
            modeladmin.admin_site.each_context(request),
            title=title,
            objects_name=objects_name,
            deletable_objects=[deletable_objects],
            model_count=dict(model_count).items(),
            queryset=queryset,
            perms_lacking=perms_needed,
            protected=protected,
            opts=opts,
            action_checkbox_name=helpers.ACTION_CHECKBOX_NAME,
            media=modeladmin.media,
        )

        request.current_app = modeladmin.admin_site.name

        # Display the confirmation page
        return TemplateResponse(request, modeladmin.delete_selected_confirmation_template or [
            "admin/%s/%s/delete_selected_confirmation.html" % (app_label, opts.model_name),
            "admin/%s/delete_selected_confirmation.html" % app_label,
            "admin/delete_selected_confirmation.html"
        ], context)

    replace_selected.short_description = \
        breadcrumb or ugettext_lazy("Replace %(verbose_name_plural)s with xls")
    return replace_selected



import os
from collections import OrderedDict
from tempfile import NamedTemporaryFile

from django.contrib.admin.utils import NestedObjects
from django.db import router
from django.utils.translation import ugettext_lazy as _

from djaxei.providers import get_workbook_impl


class Exporter:
    def __init__(self, dest=None, tmpdir=None, **kwargs):
        self.tmpdir = tmpdir
        self.dest = dest

    def xls_export(self, _models, root=None, root_qs=None):
        if (root and root_qs) or ((root or root_qs) is None):
            raise RuntimeError(_("Either a root object or a root queryset must be provided"))

        workbook = None
        workbookfile = None
        try:
            sheets = OrderedDict()

            lmodels = {}
            for k, v in _models.items():
                lname = k.lower()
                model_name = lname.rsplit('.')[1]
                lmodels[lname] = v
                sheets[model_name] = [v, ]

            if root:
                root_qs = root._meta.model.objects.filter(pk=root.pk)

            using = router.db_for_write(root_qs.first()._meta.model)
            collector = NestedObjects(using=using)
            collector.collect(root_qs)

            def callback(obj):
                fields = lmodels.get(obj._meta.label_lower, None)
                if fields:
                    sheets[obj._meta.model_name].append([getattr(obj, x) for x in fields])

            collector.nested(callback)

            Workbook = get_workbook_impl()
            workbookfile = self.dest or NamedTemporaryFile(dir=self.tmpdir, suffix=Workbook._preferred_suffix, delete=False)
            Workbook(workbookfile).write_data(sheets)

            return workbookfile.name

        except Exception as e:
            if workbook:
                if not workbookfile.closed:
                    workbookfile.close()
                if os.path.exists(workbookfile.name):
                    os.remove(workbookfile.name)
            raise e


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['djaxei', 'djaxei.migrations', 'djaxei.providers']

package_data = \
{'': ['*'], 'djaxei': ['templates/djaxei/*']}

install_requires = \
['django>=1.11,<4']

setup_kwargs = {
    'name': 'djaxei',
    'version': '0.6.1',
    'description': 'A django admin extension for importing exporting records from/to xls/ods',
    'long_description': '# A django admin extension for importing exporting records from/to xls/ods\n\nA Python library project using:\n* pytest\n* flake8\n* tox\n* bumpversion\n* isort\n\n* Free software: MIT license\n* Documentation: __TBD__\n\n\nFeatures\n--------\n\n* TODO\n\nCredits\n-------\n\n',
    'author': 'Giovanni Bronzini',
    'author_email': 'g.bronzini@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/GigiusB/djaxei.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)
